import 'dart:async';

import 'package:sudoku/cubit/game_cubit.dart';
import 'package:sudoku/models/game.dart';
import 'package:sudoku/models/types.dart';

class BoardAnimate {
  // Start game animation: blinking tiles
  static AnimatedBoardSequence createStartGameAnimationPatterns(Game game) {
    AnimatedBoardSequence patterns = [];

    int patternsCount = 3;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      AnimatedBoard pattern = [];
      for (int row = 0; row < game.boardSize; row++) {
        List<bool> patternRow = [];
        for (int col = 0; col < game.boardSize; col++) {
          patternRow.add(((patternIndex + row + col) % 2 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Win game animation: fill board with colored rows, from bottom to top
  static AnimatedBoardSequence createWinGameAnimationPatterns(Game game) {
    AnimatedBoardSequence patterns = [];

    int patternsCount = game.boardSize + 6;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      AnimatedBoard pattern = [];
      for (int row = 0; row < game.boardSize; row++) {
        List<bool> patternRow = [];
        for (int col = 0; col < game.boardSize; col++) {
          patternRow.add(row > (patternIndex - 4));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Default multi-purpose animation: sliding stripes, from top left to right bottom
  static AnimatedBoardSequence createDefaultAnimationPatterns(Game game) {
    AnimatedBoardSequence patterns = [];

    int boardSideLength = game.boardSize;
    int patternsCount = boardSideLength;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      AnimatedBoard pattern = [];
      for (int row = 0; row < game.boardSize; row++) {
        List<bool> patternRow = [];
        for (int col = 0; col < game.boardSize; col++) {
          patternRow.add(((patternIndex + row + col) % 4 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  static void startAnimation(GameCubit gameCubit, String animationType) {
    final Game game = gameCubit.state.game;

    AnimatedBoardSequence patterns = [];

    switch (animationType) {
      case 'start':
        patterns = createStartGameAnimationPatterns(game);
        break;
      case 'win':
        patterns = createWinGameAnimationPatterns(game);
        break;
      default:
        patterns = createDefaultAnimationPatterns(game);
    }

    int patternIndex = patterns.length;

    gameCubit.updateAnimationInProgress(true);

    const interval = Duration(milliseconds: 200);
    Timer.periodic(
      interval,
      (Timer timer) {
        if (patternIndex == 0) {
          timer.cancel();
          gameCubit.resetAnimatedBackground();
          gameCubit.updateAnimationInProgress(false);
        } else {
          patternIndex--;
          gameCubit.setAnimatedBackground(patterns[patternIndex]);
        }
      },
    );
  }
}
